
subroutine advect_scalar3D (f, u, v, w, rho, rhow, flux, dosubtr, fsubtr)
 	
!     positively definite monotonic advection with non-oscillatory option

use grid
use terrain, only: terra
use vars, only: misc
implicit none


real f(dimx1_s:dimx2_s, dimy1_s:dimy2_s, nzm)
real u(dimx1_u:dimx2_u, dimy1_u:dimy2_u, nzm)
real v(dimx1_v:dimx2_v, dimy1_v:dimy2_v, nzm)
real w(dimx1_w:dimx2_w, dimy1_w:dimy2_w, nz )
real rho(nzm)
real rhow(nz)
real flux(nz)
logical dosubtr ! flag to subtract minimum before advection (recommended for temperature)
real fsubtr
	
real mx (0:nxp1,0:nyp1,nzm)
real mn (0:nxp1,0:nyp1,nzm)
real uuu(-1:nxp3,-1:nyp2,nzm)
real vvv(-1:nxp2,-1:nyp3,nzm)
real www(-1:nxp2,-1:nyp2,nz)

real, parameter :: eps = 1.e-9
real g(-1:nyp2,nzm),ig(-1:nyp2,nzm)
integer i,j,k,ic,ib,jc,jb,kc,kb
logical nonos
real rhoz,dtdz


real x1, x2, a, b, a1, a2, y
real andiff,across,pp,pn,fmin
andiff(x1,x2,a,b)=(abs(a)-2.*a*a/b)*0.5*(x2-x1)
across(x1,a,a1,a2)=0.0625*x1*a1*a2/a
pp(y)= max(0.,y)
pn(y)=-min(0.,y)
	
nonos = .true.

www(:,:,nz)=0.

do j=-1,nyp2
 do k=1,nzm
  g(j,k) = rho(k)*ady(j)*adz(k)*mu(j)
  ig(j,k) = 1./g(j,k)
 end do
end do

! ------
! MK: 12/16 remove minimum for accuracy of single precision, especially for temperature

if(dosubtr) then
!  fmin=minval(f)
!  f=f-fmin
   fmin = fsubtr
else
  fmin=0.
end if


!-----------------------------------------
	 	 
if(nonos) then

 do k=1,nzm
  kc=min(nzm,k+1)
  kb=max(1,k-1)
  do j=0,nyp1
   jb=j-1
   jc=j+1
   do i=0,nxp1
    ib=i-1
    ic=i+1
    mx(i,j,k)=max(f(ib,j,k),f(ic,j,k),f(i,j,kb),f(i,j,kc),f(i,j,k),f(i,jb,k),f(i,jc,k))
    mn(i,j,k)=min(f(ib,j,k),f(ic,j,k),f(i,j,kb),f(i,j,kc),f(i,j,k),f(i,jb,k),f(i,jc,k))
   end do
  end do
 end do
	 
end if  ! nonos

 do k=1,nzm
  do j=-1,nyp2
   do i=-1,nxp3
    uuu(i,j,k)=max(0.,u(i,j,k))*(f(i-1,j,k)-fmin)+min(0.,u(i,j,k))*(f(i,j,k)-fmin)
   end do
  end do
 end do
 do k=1,nzm
  do j=-1,nyp3
   do i=-1,nxp2
    vvv(i,j,k)=max(0.,v(i,j,k))*(f(i,j-1,k)-fmin)+min(0.,v(i,j,k))*(f(i,j,k)-fmin)
   end do
  end do
 end do
 do k=1,nzm
  kb=max(1,k-1)
  do j=-1,nyp2
   do i=-1,nxp2
    www(i,j,k)=max(0.,w(i,j,k))*(f(i,j,kb)-fmin)+min(0.,w(i,j,k))*(f(i,j,k)-fmin)
   end do
  end do
  flux(k) = 0.
  do j=1,ny
   do i=1,nx
    flux(k) = flux(k) + www(i,j,k)
   end do
  end do
 end do
 if(collect_coars) misc(1:nx,1:ny,1:nzm) = www(1:nx,1:ny,1:nzm)


 do k=1,nzm
  do j=-1,nyp2
   do i=-1,nxp2
      f(i,j,k) = f(i,j,k) &
              - (uuu(i+1,j,k)-uuu(i,j,k)+vvv(i,j+1,k)-vvv(i,j,k)+ &
                www(i,j,k+1)-www(i,j,k))*ig(j,k)*terra(i,j,k)
   end do
  end do
 end do 

 do k=1,nzm
  kc=min(nzm,k+1)
  kb=max(1,k-1)
  do j=0,nyp1
   jb=j-1
   jc=j+1
   do i=0,nxp2
    ib=i-1
    uuu(i,j,k)=andiff(f(ib,j,k),f(i,j,k),u(i,j,k),g(j,k)+g(j,k)) &
              -across(f(ib,jc,k)+f(i,jc,k)-f(ib,jb,k)-f(i,jb,k), g(j,k)+g(j,k), & 
                      u(i,j,k), v(ib,j,k)+v(ib,jc,k)+v(i,jc,k)+v(i,j,k)) &
              -across(f(ib,j,kc)+f(i,j,kc)-f(ib,j,kb)-f(i,j,kb), g(j,k)+g(j,k), & 
                      u(i,j,k), w(ib,j,k)+w(ib,j,k+1)+w(i,j,k)+w(i,j,k+1))
   end do
  end do
 end do


 do k=1,nzm
  kc=min(nzm,k+1)
  kb=max(1,k-1)
  do j=0,nyp2
   jb=j-1
   do i=0,nxp1
    ib=i-1
    ic=i+1
    vvv(i,j,k)=andiff(f(i,jb,k),f(i,j,k),v(i,j,k),g(j,k)+g(jb,k)) &
              -across(f(ic,jb,k)+f(ic,j,k)-f(ib,jb,k)-f(ib,j,k), g(j,k)+g(jb,k), &
                      v(i,j,k), u(i,jb,k)+u(i,j,k)+u(ic,j,k)+u(ic,jb,k)) &
              -across(f(i,jb,kc)+f(i,j,kc)-f(i,jb,kb)-f(i,j,kb), g(j,k)+g(jb,k), & 
                      v(i,j,k), w(i,jb,k)+w(i,j,k)+w(i,j,k+1)+w(i,jb,k+1))
   end do
  end do
 end do

 do k=1,nzm
  kb=max(1,k-1)
  do j=0,nyp1
   jb=j-1
   jc=j+1
   do i=0,nxp1
    ib=i-1
    ic=i+1
    www(i,j,k)=andiff(f(i,j,kb),f(i,j,k),w(i,j,k),g(j,k)+g(j,kb)) &
             -across(f(ic,j,kb)+f(ic,j,k)-f(ib,j,kb)-f(ib,j,k), g(j,k)+g(j,kb), &
                     w(i,j,k), u(i,j,kb)+u(i,j,k)+u(ic,j,k)+u(ic,j,kb)) &
             -across(f(i,jc,k)+f(i,jc,kb)-f(i,jb,k)-f(i,jb,kb), g(j,k)+g(j,kb), &
                     w(i,j,k), v(i,j,kb)+v(i,jc,kb)+v(i,jc,k)+v(i,j,k))
   end do
  end do
 end do

www(:,:,1) = 0.

!---------- non-osscilatory option ---------------

if(nonos) then

 do k=1,nzm
  kc=min(nzm,k+1)
  do j=0,nyp1
   jc=j+1
   do i=0,nxp1
    ic=i+1
     mx(i,j,k)=(mx(i,j,k)-f(i,j,k))*g(j,k)/ &
                       (pn(uuu(ic,j,k)) + pp(uuu(i,j,k))+ &
                       (pn(vvv(i,jc,k)) + pp(vvv(i,j,k)))+ &
                       (pn(www(i,j,kc)) + pp(www(i,j,k)))+eps)
     mn(i,j,k)=(f(i,j,k)-mn(i,j,k))*g(j,k)/ &
                       (pp(uuu(ic,j,k)) + pn(uuu(i,j,k))+ &
                       (pp(vvv(i,jc,k)) + pn(vvv(i,j,k)))+ &
                       (pp(www(i,j,kc)) + pn(www(i,j,k)))+eps)	
   end do
  end do
 end do


 do k=1,nzm
  do j=1,ny
   do i=1,nxp1
    ib=i-1
    uuu(i,j,k)=pp(uuu(i,j,k))*min(1.,mx(i,j,k), mn(ib,j,k)) &
             - pn(uuu(i,j,k))*min(1.,mx(ib,j,k),mn(i,j,k))
   end do
  end do
 end do

 do k=1,nzm
  do j=1,nyp1
   jb=j-1
   do i=1,nx
    vvv(i,j,k)=pp(vvv(i,j,k))*min(1.,mx(i,j,k), mn(i,jb,k)) &
             - pn(vvv(i,j,k))*min(1.,mx(i,jb,k),mn(i,j,k))
   end do
  end do
 end do

 do k=1,nzm
  kb=max(1,k-1)
  do j=1,ny
   do i=1,nx
    www(i,j,k)=pp(www(i,j,k))*min(1.,mx(i,j,k), mn(i,j,kb)) &
             - pn(www(i,j,k))*min(1.,mx(i,j,kb),mn(i,j,k))
    flux(k) = flux(k) + www(i,j,k)
   end do
  end do
 end do


if(collect_coars) then
  dtdz = dtn/dz
  do k=1,nzm
    do j=1,ny
     rhoz = 1./(dtdz*ady(j)*mu(j))
     do i=1,nx
       misc(i,j,k) = (misc(i,j,k)+www(i,j,k))*rhoz
     end do
    end do
  end do
end if

endif ! nonos

if(dosubtr) then
  do k=1,nzm
   kc=k+1
   do j=1,ny
    do i=1,nx
     f(i,j,k)=f(i,j,k) - (uuu(i+1,j,k)-uuu(i,j,k)+vvv(i,j+1,k)-vvv(i,j,k)+ &
                      www(i,j,k+1)-www(i,j,k))*ig(j,k)*terra(i,j,k)
    end do
   end do
  end do 
else
  do k=1,nzm
   kc=k+1
   do j=1,ny
    do i=1,nx
     f(i,j,k)=max(0.,f(i,j,k) - (uuu(i+1,j,k)-uuu(i,j,k)+vvv(i,j+1,k)-vvv(i,j,k)+ &
                      www(i,j,k+1)-www(i,j,k))*ig(j,k)*terra(i,j,k))
    end do
   end do
  end do 
end if

end subroutine advect_scalar3D


